//+------------------------------------------------------------------+
//|                                       Linear Regression Line.mq4 |
//|                                                      MQL Service |
//|                                           scripts@mqlservice.com |
//+------------------------------------------------------------------+
#property copyright "MQL Service"
#property link      "www.mqlservice.com"

#property indicator_chart_window
#property indicator_buffers   1
#property indicator_color1    Orange
#property indicator_width1    2

//---- input parameters
extern int LRLPeriod = 20;

//---- buffers
double LRLBuffer[];


//+------------------------------------------------------------------+
//|                    INITIALIZATION FUNCTION                       |
//+------------------------------------------------------------------+
int init()
{
	//---- indicators
	SetIndexStyle(0, DRAW_LINE);
	SetIndexBuffer(0, LRLBuffer);
	IndicatorDigits(Digits);
	if (LRLPeriod < 2) 
		LRLPeriod = 2;
		
	IndicatorShortName("Linear Regression Line ("+LRLPeriod+")");
	SetIndexDrawBegin(0, LRLPeriod+2);
	IndicatorDigits(MarketInfo(Symbol(), MODE_DIGITS)+4);

	return(0);
}

//+------------------------------------------------------------------+
//|                   DEINITIALIZATION FUNCTION                      |
//+------------------------------------------------------------------+
int deinit()
{
	return(0);
}

//+------------------------------------------------------------------+
//|                      ITERATION FUNCTION                          |
//+------------------------------------------------------------------+
int start()
{
	int limit;
	int counted_bars = IndicatorCounted();

	if (counted_bars < 0) 
		counted_bars = 0;
  
	if (counted_bars > 0) 
		counted_bars--;
  
	limit = Bars - counted_bars;

/*+------------------------------------------------------------------+
http://phoenix.phys.clemson.edu/tutorials/excel/regression.html

*/
double sumx,sumy,sumxy,sumx2,sumy2,yint,slope,r;

	for (int shift=limit-1; shift >= 0; shift--)
	{
		sumx = 0;
		sumy = 0;
		sumxy = 0;
		sumx2 = 0;
		sumy2 = 0;
		for (int n = 0; n <= LRLPeriod-1; n++)
		{ 
			sumx = sumx + n;                                                           //sum(X)
			sumy = sumy + Close[shift + n];                                            //sum(Y)
			sumxy = sumxy + n * Close[shift + n];                                      //sum(X*Y)
			sumx2 = sumx2 + n * n;                                                     //sum(X^2)
			sumy2 = sumy2 + Close[shift + n] * Close[shift + n];                       //sum(Y^2)
		}

//          N*sum(X*Y)-sum(X)*sum(Y)/N*sum(X^2)-sum(X)^2
		slope = (LRLPeriod * sumxy - sumx * sumy) / (LRLPeriod * sumx2 - sumx * sumx);

//          sum(Y)-slope*sum(X)/N
		yint = (sumy - slope * sumx) / LRLPeriod; // Y Intercept

//          N*sum(XY)-sum(X)*sum(Y)/SQRT((N*sum(X^2)-sum(X)^2-sum(X)^2)*(N*sum(Y^2)-sum(Y)^2))
		r = (LRLPeriod * sumxy - sumx * sumy) / MathSqrt((LRLPeriod * sumx2 - sumx * sumx) * (LRLPeriod * sumy2 - sumy * sumy)); // correlation coefficient

//                  y  = slope*x+yint         y is the price level x is the bsr bieng plotted or 1 (left in for clarity)
		LRLBuffer[shift] = slope*1+yint;

	}
	return(0);
}
//+------------------------------------------------------------------+